// Game.cpp : Defines the entry point for the application.
//

#include	"stdafx.h"
#include	"resource.h"
#include	"..\Common\Common.h"
#include	"..\Display\Common.h"
#include	"..\Input\Input1.h"
#include	"..\Engine\Sprite.h"
//
// Globalni handle na okno
HWND		g_hWnd;
//
// Prototypy globalnich fci
LRESULT CALLBACK MainWndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
HRESULT WinInit(HINSTANCE hInstance);
void	UpdateFrame();

#define		_S_DATAFILE		_T("Data.dat")
#define		_S_BACKGROUND	_T("\\Graphics\\Backgrounds\\mars800.bmp")

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
 	HRESULT dwRet;
	MSG msg; 

	dwRet = WinInit(hInstance);
	if(dwRet != ERROR_SUCCESS) {
		DXTRACE("Nemohu vytvorit okno aplikace.");
		return dwRet;
	}

	CString csDataPath = setGetDataFilePath(_S_DATAFILE);
	dwRet = stgOpenDataFile3(csDataPath, NULL);
	if(dwRet != ERROR_SUCCESS) {
		DXERR("Cannot open data file due", dwRet);
		return dwRet;
	}

	disInit(g_hWnd, DDFLG_CLIPPER|DDFLG_FULLSCREEN);
	inpCreateDirectInputSystem(hInstance, g_hWnd, disGetResolution());

	disDefineBackground(_S_BACKGROUND, 0);
	// Run message loop
	while( TRUE )
    {
        // Look for messages, if none are found then 
        // update the state and display it
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            if( 0 == GetMessage(&msg, NULL, 0, 0 ) )
            {
                // WM_QUIT was posted, so exit
                return (int)msg.wParam;
            }
			
            // Translate and dispatch the message
            TranslateMessage( &msg ); 
            DispatchMessage( &msg );
        }
        else
        {
			UpdateFrame();			
		}
    }	

	return 0;
}

HRESULT WinInit(HINSTANCE hInstance) 
{
    DWORD dwRet;
    WNDCLASSEX wc; 
    //
    // Register the Window Class
    wc.cbSize = sizeof(WNDCLASSEX);
    wc.lpszClassName = "GameWin";
    wc.lpfnWndProc = MainWndProc;
    wc.style = CS_VREDRAW|CS_HREDRAW;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance,MAKEINTRESOURCE(IDI_PROGRAM_ICON));
    wc.hCursor = NULL;
    wc.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
    wc.lpszMenuName = NULL;    
    wc.hIconSm = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_PROGRAM_ICON));
	wc.cbClsExtra    = 0;
	wc.cbWndExtra	 = 0;
    //
    // Register window class
    if(RegisterClassEx(&wc) == 0) {
        dwRet = GetLastError();
        DXERR("Cannot register the window class due", dwRet); 
        return dwRet; 
    } 
    //
    // Create new main window
    g_hWnd = CreateWindow("GameWin", "Game", 
                            WS_OVERLAPPEDWINDOW, 0, 0, 200, 
                            200, NULL, NULL, hInstance, NULL);
    if(!g_hWnd) {
        dwRet = GetLastError();
        DXERR("Cannot create the window due", dwRet); 
        return dwRet;
    }
    ShowWindow(g_hWnd, SW_SHOW);
    UpdateWindow(g_hWnd);
    dwRet = ERROR_SUCCESS;
    return dwRet;
}

LRESULT CALLBACK MainWndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch(msg) {
	case WM_SETCURSOR:
		SetCursor(NULL);		
		return 0L;
	case WM_SYSCOMMAND:
        // Prevent moving/sizing and power loss in fullscreen mode
        switch( wParam )
        {
            case SC_MOVE:
            case SC_SIZE:
            case SC_MAXIMIZE:
            case SC_KEYMENU:
            case SC_MONITORPOWER:
                return 1;                
        }
		break;
	case WM_DESTROY:
        // Cleanup and close the app            
        PostQuitMessage( 0 );
        return 0L;
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}

void	UpdateFrame()
{
	disUpdateBackground();

	inpProcessInput();

	// Pri stisknuti klavesy Esc ukoncime aplikaci
	if(inpIsKeyDown(DIK_ESCAPE, FALSE)) {
		PostMessage(g_hWnd, WM_DESTROY, 0, 0);
	}

	inpUpdateCursor();

	disPresent();
}